' Copyright (c) 2007-2007 Quadralay Corporation.  All rights reserved.
'
'$Id: wwh4merge.vbs,v 1.3 2006/03/22 22:17:02 jwiles Exp $
'

' ###############################################

GlobalAutoMapExecutablePath = "C:\Program Files\WebWorks\ePublisher AutoMap\WebWorks.Automap.exe"
GlobalJarPath = "C:\j2sdk1.4.2_14\bin\jar.exe"

' ###############################################

Set GlobalFileSystem = CreateObject("Scripting.FileSystemObject")
Set GlobalShell = CreateObject("WScript.Shell")
GlobalProjectNamespace = "urn:WebWorks-Publish-Project"

' Force non-interactive operation
'
WScript.Interactive = False

WScript.Quit(Main)

' ###############################################

Function  WW_PadWithPrefix(ParamString, _
                           ParamPrefix, _
                           ParamMinLength)
  Dim VarPaddedWithPrefix

  VarPaddedWithPrefix = ParamString
  While (Len(VarPaddedWithPrefix) < ParamMinLength)
    VarPaddedWithPrefix = ParamPrefix & VarPaddedWithPrefix
  Wend

  WW_PadWithPrefix = VarPaddedWithPrefix
End Function

' ###############################################

Function WW_FormattedDate(ParamDate)
  Dim VarFormattedDate

  VarFormattedDate = "" & WW_PadWithPrefix(DatePart("yyyy", ParamDate), "0", 4)
  VarFormattedDate = VarFormattedDate & "-" & WW_PadWithPrefix(DatePart("m", ParamDate), "0", 2)
  VarFormattedDate = VarFormattedDate & "-" & WW_PadWithPrefix(DatePart("d", ParamDate), "0", 2)

  WW_FormattedDate = VarFormattedDate
End Function

' ###############################################

Function WW_FormattedTime(ParamTime)
  Dim VarFormattedTime

  VarFormattedTime = "" & WW_PadWithPrefix(DatePart("h", ParamTime), "0", 2)
  VarFormattedTime = VarFormattedTime & "-" & WW_PadWithPrefix(DatePart("n", ParamTime), "0", 2)
  VarFormattedTime = VarFormattedTime & "-" & WW_PadWithPrefix(DatePart("s", ParamTime), "0", 2)

  WW_FormattedTime = VarFormattedTime
End Function

' ###############################################

Function WW_StringToPathComponent(ParamString)
  Dim VarPathComponent
  Dim VarMaxIndex
  Dim VarIndex
  Dim VarCharacter

  ' Initialize return value
  '
  VarValidPathComponent = ""

  VarMaxIndex = Len(ParamString)
  For VarIndex = 1 To VarMaxIndex
    VarCharacter = Mid(ParamString, VarIndex, 1)

    Select Case VarCharacter
      Case " ":
        VarCharacter = "_"

      Case "\", _
           "/", _
           ":", _
           "*", _
           "?", _
           """", _
           "<", _
           ">", _
           "|":
        VarCharacter = "_"
    End Select

    VarPathComponent = VarPathComponent & VarCharacter
  Next

  WW_StringToPathComponent = VarPathComponent
End Function

' ###############################################

Sub WW_Log(ParamMessage)
  Call WScript.Echo(ParamMessage)
End Sub

' ###############################################

Function WW_GetTempFilePath()
  Dim VarTempFileName
  Dim VarTempFolderPath
  Dim VarTempFilePath

  VarTempFolderPath = GlobalFileSystem.GetSpecialFolder(2)  ' 2 == TemporaryFolder
  VarTempFileName = GlobalFileSystem.GetTempName
  VarTempFilePath = GlobalFileSystem.BuildPath(VarTempFolderPath, VarTempFileName)

  WW_GetTempFilePath = VarTempFilePath
End Function

' ###############################################

Function WW_Execute(ParamCommandLine, _
                    ParamLog)
  Dim VarReturnCode
  Dim VarLogFilePath
  Dim VarBatchFilePath
  Dim VarBatchFile
  Dim VarLogFile

  ' Initialize return value
  '
  VarReturnCode = -1

  ' Need to create a temporary batch file for this to work
  '
  VarLogFilePath = WW_GetTempFilePath()
  VarBatchFilePath = WW_GetTempFilePath() & ".bat"
  Set VarBatchFile = GlobalFileSystem.CreateTextFile(VarBatchFilePath, True, False)
  Call VarBatchFile.WriteLine(ParamCommandLine & " >> " & """" & VarLogFilePath & """ 2>&1")
  Call VarBatchFile.Close()

  ' Execute command
  ' Last two parameters of the Run method are:
  '   - Window Display mode
  '   - bWaitOnReturn
  '
  VarReturnCode = GlobalShell.Run(VarBatchFilePath, 0, True)

  ' Delete temporary batch file
  '
  Call GlobalFileSystem.DeleteFile(VarBatchFilePath)

  ' Log result
  '
  If (ParamLog) Then
    Set VarLogFile = GlobalFileSystem.OpenTextFile(VarLogFilePath, 1)
    If (Not VarLogFile.AtEndOfStream) Then
      Call WW_Log(VarLogFile.ReadAll)
    End If
    Call VarLogFile.Close()
  End If

  ' Delete temporary log file
  '
  Call GlobalFileSystem.DeleteFile(VarLogFilePath)

  WW_Execute = VarReturnCode
End Function

' ###############################################

Sub WW_CreateDirectory(ParamDirectoryPath)
  Dim VarParentDirectoryPath

  If (Not GlobalFileSystem.FolderExists(ParamDirectoryPath)) Then
    VarParentDirectoryPath = GlobalFileSystem.GetParentFolderName(ParamDirectoryPath)

    If (Len(VarParentDirectoryPath) > 0) Then
      WW_CreateDirectory(VarParentDirectoryPath)

      Call GlobalFileSystem.CreateFolder(ParamDirectoryPath)
    End If
  End If
End Sub

' ###############################################

Function GetMapTitle(ParamMapFilePath)
  Dim VarMapTitle
  Dim VarRootElement
  Dim VarMapElement

  ' Initialize return value
  '
  VarMapTitle = ""

  ' Load map
  '
  Set VarMapDocument = CreateObject("Microsoft.XMLDOM")
  VarMapDocument.async = False
  VarMapDocument.resolveExternals = False
  VarMapDocument.validateOnParse = False
  Call VarMapDocument.load(ParamMapFilePath)
  If (VarMapDocument.parseError.errorCode = 0) Then
    ' Access map element
    '
    Set VarMapElement = VarMapDocument.documentElement

    ' Access title attribute
    '
    VarMapTitle = VarMapElement.getAttribute("title")
  Else
    ' Report error
    '
    Call WW_Log("VarMapDocument.parseError.errorCode == " & VarMapDocument.parseError.errorCode)
    Call WW_Log("VarMapDocument.parseError.reason == " & VarMapDocument.parseError.reason)
  End If

  GetMapTitle = VarMapTitle
End Function

' ###############################################

Function WriteProjectFile(ParamProjectFilePath, _
                          ParamStationeryFilePath, _
                          ParamGroupName, _
                          ParamDocumentFilePath)
  Dim VarSuccess
  Dim VarProjectDocument
  Dim VarProjectElement
  Dim VarOriginElement
  Dim VarPathElement
  Dim VarGroupsElement
  Dim VarGroupElement
  Dim VarDocumentElement

  ' Initialize return value
  '
  VarSuccess = False

  ' Create project directory
  '
  Call WW_CreateDirectory(GlobalFileSystem.GetParentFolderName(ParamProjectFilePath))

  ' Create project document
  '
  Set VarProjectDocument = CreateObject("Microsoft.XMLDOM")

  ' Project element
  '
  Set VarProjectElement = VarProjectDocument.createNode(1, "Project", GlobalProjectNamespace)
  Call VarProjectDocument.appendChild(VarProjectElement)
  Call VarProjectElement.setAttribute("Version", "1.1")

  ' Origin element
  '
  Set VarOriginElement = VarProjectDocument.createNode(1, "Origin", GlobalProjectNamespace)
  Call VarProjectElement.appendChild(VarOriginElement)

  ' Path element
  '
  Set VarPathElement = VarProjectDocument.createNode(1, "Path", GlobalProjectNamespace)
  Call VarOriginElement.appendChild(VarPathElement)
  VarPathElement.text = ParamStationeryFilePath

  ' Groups element
  '
  Set VarGroupsElement = VarProjectDocument.createNode(1, "Groups", GlobalProjectNamespace)
  Call VarProjectElement.appendChild(VarGroupsElement)

  ' Group element
  '
  Set VarGroupElement = VarProjectDocument.createNode(1, "Group", GlobalProjectNamespace)
  Call VarGroupsElement.appendChild(VarGroupElement)
  Call VarGroupElement.setAttribute("Name", ParamGroupName)
  Call VarGroupElement.setAttribute("GroupID", "1")

  ' Document element
  '
  Set VarDocumentElement = VarProjectDocument.createNode(1, "Document", GlobalProjectNamespace)
  Call VarGroupElement.appendChild(VarDocumentElement)
  Call VarDocumentElement.setAttribute("Type", "normal")
  Call VarDocumentElement.setAttribute("DocumentID", "2")
  Call VarDocumentElement.setAttribute("Path", ParamDocumentFilePath)

  ' Write project document
  '
  Call VarProjectDocument.save(ParamProjectFilePath)

  ' Success!
  '
  VarSuccess = True

  WriteProjectFile = VarSuccess
End Function

' ###############################################

Function ExecuteAutoMap(ParamProjectFilePath, _
                        ParamDeployFolderPath)
  Dim VarReturnCode
  Dim VarAutoMapExecutablePath
  Dim VarAutoMapCommand

  ' Initialize return value
  '
  VarReturnCode = -1

  ' Determine AutoMap executable path
  '
  VarAutoMapExecutablePath = GlobalAutoMapExecutablePath

  ' Build AutoMap command
  '
  VarAutoMapCommand = """" & VarAutoMapExecutablePath & """ -d """ & ParamDeployFolderPath & """ """ & ParamProjectFilePath & """"

  ' Run AutoMap
  '
  Call WW_Log(VarAutoMapCommand)
  VarReturnCode = WW_Execute(VarAutoMapCommand, True)

  ExecuteAutoMap = VarReturnCode
End Function

' ###############################################

Function ZipDirectory(ParamDirectoryPath, _
                      ParamZipPath)
  Dim VarReturnCode
  Dim VarJarPath
  Dim VarParentDirectoryPath
  Dim VarDirectoryName

  ' Initialize return value
  '
  VarReturnCode = -1

  ' Source directory exists?
  '
  If (GlobalFileSystem.FolderExists(ParamDirectoryPath)) Then
    ' Determine jar command location
    '
    VarJarPath = GlobalJarPath

    ' Create zip directory
    '
    Call WW_CreateDirectory(GlobalFileSystem.GetParentFolderName(ParamZipPath))

    ' Build Zip command
    '
    VarParentDirectoryPath = GlobalFileSystem.GetParentFolderName(ParamDirectoryPath)
    VarDirectoryName = GlobalFileSystem.GetFileName(ParamDirectoryPath)
    VarZipCommand = """" & VarJarPath & """ -cfM """ & ParamZipPath & """ -C """ & VarParentDirectoryPath & """ """ & VarDirectoryName & """"

    ' Run Zip command
    '
    Call WW_Log(VarZipCommand)
    VarReturnCode = WW_Execute(VarZipCommand, True)
  End If

  ZipDirectory = VarReturnCode
End Function

' ###############################################

Function Main()
  Dim VarReturnCode
  Dim VarDocumentFilePath
  Dim VarStationeryFilePath
  Dim VarProjectFileWritten
  Dim VarArchiveDirectoryPath
  Dim VarMapTitle
  Dim VarMapTitlePathComponent
  Dim VarNow
  Dim VarArchiveName
  Dim VarProjectDirectoryPath
  Dim VarProjectAndGroupName
  Dim VarProjectFilePath
  Dim VarDeployFolderPath
  Dim VarZipSourceDirectoryPath
  Dim VarZipPath

  ' Initialize return value
  '
  VarReturnCode = -1

  ' Check arguements
  '
  If (WScript.Arguments.Count = 3) Then
    ' Parse arguments
    '
    VarDocumentFilePath = GlobalFileSystem.GetAbsolutePathname(WScript.Arguments(0))
    VarStationeryFilePath = GlobalFileSystem.GetAbsolutePathname(WScript.Arguments(1))
    VarArchiveDirectoryPath = GlobalFileSystem.GetAbsolutePathname(WScript.Arguments(2))

    ' Determine archive name
    '
    VarMapTitle = GetMapTitle(VarDocumentFilePath)
    VarMapTitlePathComponent = WW_StringToPathComponent(Trim(VarMapTitle))
    If (Len(VarMapTitlePathComponent) > 32) Then
      VarMapTitlePathComponent = Left(VarMapTitlePathComponent, 32)
    End If
    VarNow = Now()
    VarArchiveName = VarMapTitlePathComponent & "_" & WW_FormattedDate(VarNow) & "_" & WW_FormattedTime(VarNow)

    ' Determine base project/group name along with full project path
    '
    VarProjectDirectoryPath = WW_GetTempFilePath()
    VarProjectAndGroupName = VarArchiveName
    VarProjectFilePath = GlobalFileSystem.BuildPath(VarProjectDirectoryPath, VarProjectAndGroupName & ".wrp")

    ' Write project file
    '
    VarProjectFileWritten = WriteProjectFile(VarProjectFilePath, _
                                             VarStationeryFilePath, _
                                             VarProjectAndGroupName, _
                                             VarDocumentFilePath)
    If (VarProjectFileWritten) Then
      ' Execute AutoMap
      '
      VarDeployFolderPath = WW_GetTempFilePath()
      VarReturnCode = ExecuteAutoMap(VarProjectFilePath, VarDeployFolderPath)
      If (VarReturnCode = 0) Then
        ' Zip result
        '
        VarZipSourceDirectoryPath = GlobalFileSystem.BuildPath(VarDeployFolderPath, VarArchiveName)
        VarZipPath = GlobalFileSystem.BuildPath(VarArchiveDirectoryPath, VarArchiveName & ".zip")
        VarReturnCode = ZipDirectory(VarZipSourceDirectoryPath, VarZipPath)

        ' Clean up
        '
        Call GlobalFileSystem.DeleteFolder(VarDeployFolderPath)
      End If

      ' Clean up
      '
      Call GlobalFileSystem.DeleteFolder(VarProjectDirectoryPath)
    End If
  Else
    ' Report usage
    '
    Call WW_Log("Usage:")
    Call WW_Log("  epublish.vbs <topic-map-file-path> <stationery-path> <archive-directory-path>")
  End If

  Main = VarReturnCode
End Function

' ###############################################
